//
//  SWCell.m
//  White Detector Grid
//
//  Created by David Langford on 9/05/09.
//  Copyright 2009 Sentient Worlds. All rights reserved.
//

#import "SWCell.h"
#import <OpenGL/CGLMacro.h>

@implementation SWCell

@synthesize xInPixels
			, yInPixels
			, widthInPixels
			, heightInPixels
			, weight;

- (id) initWithXPositionAt:(NSNumber*)startXposition
			   yPositionAt:(NSNumber*)startYposition
			 widthInPixels:(NSNumber*)startWidth
			heightInPixels:(NSNumber*)startHeight
{
	if (self = [super init])
	{
		// NSNumber* that are copy need to be released
		self.xInPixels = [startXposition copy];
		self.yInPixels = [startYposition copy];
		self.widthInPixels = [startWidth copy];
		self.heightInPixels = [startHeight copy];
	}
	return self;
}

- (void) dealloc
{
	[self.xInPixels release];
	[self.yInPixels release];
	[self.widthInPixels release];
	[self.heightInPixels release];
	[super dealloc];
}

// Generates the weight for the cell
- (BOOL) generateWeightFromImage:(id<QCPlugInInputImageSource>)imageToUse
					   inContext:(id<QCPlugInContext>)theContext
{
	// Detect the white in this image
	vImage_Buffer					buffer;
	vImage_Error					error;
	vImagePixelCount*				histograms[4];
	
	if (imageToUse) {

		// Generate buffer depending on where this cell exists relative to the image
		int pixelSizeInBytes = 4;
		buffer.width = [self.widthInPixels intValue];
		buffer.height = [self.heightInPixels intValue];
		buffer.rowBytes = [imageToUse bufferBytesPerRow];
		buffer.data = (void*)([imageToUse bufferBaseAddress]					// Start at the image root
							  + (buffer.rowBytes*[self.yInPixels intValue])		// Go to the correct row
							  + (pixelSizeInBytes*([self.xInPixels intValue]))	// Go to the correct pixel
							  );
		
		// Generate the buffer
		histograms[0] = _histogramA;
		histograms[1] = _histogramR;
		histograms[2] = _histogramG;
		histograms[3] = _histogramB;
		error = vImageHistogramCalculation_ARGB8888(&buffer, histograms, 0);
		
		if (error != kvImageNoError)
			return NO;
	}
	
	// Store buffer values
	histograms[0] = _histogramR;
	histograms[1] = _histogramG;
	histograms[2] = _histogramB;
	histograms[3] = _histogramA;
	
	// Generate the weight
	self.weight = [self generateWeightFromHistogram:histograms
									   usingChannel:1];
	
	return YES;
}

// For now just find what intensity has the highest amount of pixels
// Note: This is the wrong equation for what I need, but will work for the moment
-(NSNumber*) generateWeightFromHistogram:(vImagePixelCount**)histograms
							usingChannel:(int)channel;
{
	int numberOfIntensityChannels = 256;
	int currentHighestChannel = 0;
	int currentHighestNumberOfPixels = 0;
	
	for (int i=0; i<numberOfIntensityChannels; i++) {
		int numberOfPixels = histograms[channel][i];
		if (numberOfPixels > currentHighestNumberOfPixels)
		{
			currentHighestChannel = i;
			currentHighestNumberOfPixels = numberOfPixels;
		}
	}
	
	return [NSNumber numberWithFloat:currentHighestChannel/255.0];
}


// Returns an NSArray representing itself
- (NSArray*) asNSArray
{
	return [NSArray arrayWithObjects: self.xInPixels, self.yInPixels, self.widthInPixels, self.heightInPixels, self.weight, nil];
}

@end
