//
//  White_Detector_GridPlugIn.m
//  White Detector Grid
//
//  Created by David Langford on 1/05/09.
//  Copyright (c) 2009 Sentient Worlds. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "White_Detector_GridPlugIn.h"
#import "SWGrid.h"

#define	kQCPlugIn_Name				@"White Detector Grid"
#define	kQCPlugIn_Description		@"Detects white using a grid format"

@implementation White_Detector_GridPlugIn

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputImage
		, outputArray
		, inputNumberOfColumns
		, inputNumberOfRows
		, inputWidthInPercent
		, inputHeightInPercent
		, inputXOriginInPercent
		, inputYOriginInPercent;

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	if([key isEqualToString:@"inputImage"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Source Image", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputNumberOfColumns"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Number of columns", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputNumberOfRows"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Number of Rows", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"outputArray"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Weights", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputWidthInPercent"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Width of grid (%)", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputHeightInPercent"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Height of gird (%)", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputXOriginInPercent"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"X Origin (%)", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputYOriginInPercent"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Y Origin (%)", QCPortAttributeNameKey, nil];
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	
	[super dealloc];
}

@end

@implementation White_Detector_GridPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	if (self.inputImage)
	{
		
		// Create the grid
		SWGrid* grid = [[SWGrid alloc] initWithNumberOfColumns: [NSNumber numberWithFloat:self.inputNumberOfColumns]
												  numberOfRows: [NSNumber numberWithFloat:self.inputNumberOfRows]
													imageToUse: self.inputImage
												   withContext: context
												  widthPercent: [NSNumber numberWithFloat:self.inputWidthInPercent]
												 heightPercent: [NSNumber numberWithFloat:self.inputHeightInPercent]
												xOriginPercent: [NSNumber numberWithFloat:self.inputXOriginInPercent]
												yOriginPercent: [NSNumber numberWithFloat:self.inputYOriginInPercent]];
		// Output this grid
		self.outputArray = [grid asNSArray];
		
		// Don't need the grid anymore so release it
		[grid release];
	}
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
